@@ -1096,6 +1096,7 @@ struct ospf_interface *add_ospf_interface(struct connected *co,
 	oi->type = IF_DEF_PARAMS(co->ifp)->type;
 	oi->ptp_dmvpn = IF_DEF_PARAMS(co->ifp)->ptp_dmvpn;
 	oi->p2mp_delay_reflood = IF_DEF_PARAMS(co->ifp)->p2mp_delay_reflood;
+	oi->p2mp_non_broadcast = IF_DEF_PARAMS(co->ifp)->p2mp_non_broadcast;
 
 	/* Add pseudo neighbor. */
 	ospf_nbr_self_reset(oi, oi->ospf->router_id);
@@ -1989,7 +1990,7 @@ static void ospf_nbr_nbma_add(struct ospf_nbr_nbma *nbr_nbma,
 	struct route_node *rn;
 	struct prefix p;
 
-	if (oi->type != OSPF_IFTYPE_NBMA)
+	if (!OSPF_IF_NON_BROADCAST(oi))
 		return;
 
 	if (nbr_nbma->nbr != NULL)
@@ -2036,7 +2037,7 @@ void ospf_nbr_nbma_if_update(struct ospf *ospf, struct ospf_interface *oi)
 	struct route_node *rn;
 	struct prefix_ipv4 p;
 
-	if (oi->type != OSPF_IFTYPE_NBMA)
+	if (!OSPF_IF_NON_BROADCAST(oi))
 		return;
 
 	for (rn = route_top(ospf->nbr_nbma); rn; rn = route_next(rn))
@@ -2095,7 +2096,7 @@ int ospf_nbr_nbma_set(struct ospf *ospf, struct in_addr nbr_addr)
 	rn->info = nbr_nbma;
 
 	for (ALL_LIST_ELEMENTS_RO(ospf->oiflist, node, oi)) {
-		if (oi->type == OSPF_IFTYPE_NBMA)
+		if (OSPF_IF_NON_BROADCAST(oi))
 			if (prefix_match(oi->address, (struct prefix *)&p)) {
 				ospf_nbr_nbma_add(nbr_nbma, oi);
 				break;
